"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Errors_1 = require("./../utils/Errors");
const Job_1 = require("./../entities/Job");
class FirebaseRedundancyService {
    constructor(reference) {
        if (!reference) {
            throw new Error(Errors_1.Errors.INVALID_ROOT_REFERENCE);
        }
        this.redundancyNodeReference = reference;
    }
    deserialize(data) {
        switch (data['type']) {
            case Job_1.Job.TASK_TYPE:
                return Job_1.Job.fromJson(data);
            default:
                return null;
        }
    }
    getRef() {
        return this.redundancyNodeReference;
    }
    getAll() {
        return __awaiter(this, void 0, void 0, function* () {
            const nodeSnapshot = yield this.redundancyNodeReference.once('value');
            if (!nodeSnapshot.exists()) {
                return null;
            }
            const tasks = new Array();
            nodeSnapshot.forEach((jobSnapshot) => {
                if (!jobSnapshot.exists()) {
                    return false;
                }
                tasks.push(this.deserialize(jobSnapshot.val()));
                return false;
            });
            return tasks;
        });
    }
    fetch(key) {
        return __awaiter(this, void 0, void 0, function* () {
            const taskSnapshot = yield this.redundancyNodeReference.child(key).once('value');
            if (!taskSnapshot.exists()) {
                return null;
            }
            return this.deserialize(taskSnapshot.val());
        });
    }
    commit(task) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.redundancyNodeReference
                .child(task.getKey())
                .set(task.asJson());
            return;
        });
    }
    remove(key) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.redundancyNodeReference.child(key).remove();
            return;
        });
    }
}
exports.FirebaseRedundancyService = FirebaseRedundancyService;
