"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Errors_1 = require("./../utils/Errors");
const FirebaseRedundancyService_1 = require("./FirebaseRedundancyService");
const NodeSchedule = require("node-schedule");
class Scheduler {
    constructor(reference, taskCompletionHandler) {
        this.redundancyService = new FirebaseRedundancyService_1.FirebaseRedundancyService(reference);
        this.taskCompletionHandler = taskCompletionHandler;
        this.taskList = new Map();
        this.redundancyService.getAll().then((tasks) => {
            this.queueExisting(tasks);
        }).catch((error) => {
            throw error;
        });
    }
    getJobCallback(key) {
        return () => __awaiter(this, void 0, void 0, function* () {
            if (!this.taskCompletionHandler) {
                return;
            }
            const task = yield this.redundancyService.fetch(key);
            if (!task) {
                return;
            }
            if (this.taskCompletionHandler) {
                this.taskCompletionHandler(key, task);
            }
            yield this.redundancyService.remove(key);
        });
    }
    scheduleTask(task) {
        if (this.taskList.has(task.getKey())) {
            return;
        }
        const scheduledJob = NodeSchedule.scheduleJob(task.getKey(), task.getScheduledDateTime(), this.getJobCallback(task.getKey()));
        this.taskList.set(task.getKey(), scheduledJob);
    }
    queueExisting(tasks) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!tasks) {
                return;
            }
            tasks.forEach((task) => {
                if (task.getScheduledDateTime().getTime() < Date.now()) {
                    this.taskCompletionHandler(task.getKey(), task);
                    this.redundancyService.remove(task.getKey());
                }
                else {
                    this.scheduleTask(task);
                }
            });
        });
    }
    get(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.redundancyService.fetch(key);
        });
    }
    getPendingCount() {
        return this.taskList.size;
    }
    schedule(task) {
        return __awaiter(this, void 0, void 0, function* () {
            if (task.getScheduledDateTime().getTime() < Date.now()) {
                throw new Error(Errors_1.Errors.SCHEDULED_IN_PAST);
            }
            let shouldCreateLocallyOnly = false;
            const existingTask = yield this.redundancyService
                .fetch(task.getKey());
            if (existingTask) {
                if (this.taskList.has(task.getKey())) {
                    return existingTask;
                }
                shouldCreateLocallyOnly = true;
            }
            this.scheduleTask(task);
            if (!shouldCreateLocallyOnly) {
                yield this.redundancyService.commit(task);
            }
            return task;
        });
    }
    cancel(key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.taskList.has(key)) {
                return;
            }
            yield this.redundancyService.remove(key);
            const job = this.taskList.get(key);
            if (job) {
                job.cancel();
            }
            this.taskList.delete(key);
        });
    }
}
exports.Scheduler = Scheduler;
