"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Errors_1 = require("./../utils/Errors");
class Job {
    constructor(key, tag, datetime) {
        if (!key || key.length === 0) {
            throw new Error(Errors_1.Errors.INVALID_TASK_KEY);
        }
        if (!tag || tag.length === 0) {
            throw new Error(Errors_1.Errors.INVALID_TASK_TYPE);
        }
        if (!datetime) {
            throw new Error(Errors_1.Errors.INVALID_TASK_DATE);
        }
        this.key = key;
        this.tag = tag;
        this.scheduledDateTime = datetime;
    }
    getKey() {
        return this.key;
    }
    getTag() {
        return this.tag;
    }
    getType() {
        return Job.TASK_TYPE;
    }
    getScheduledDateTime() {
        return this.scheduledDateTime;
    }
    getPayload() {
        return this.payload;
    }
    setPayload(payload) {
        this.payload = payload;
    }
    static fromJson(data) {
        const job = new Job(data['id'], data['tag'], new Date(data['scheduled_date_time']));
        if (data['payload']) {
            job.setPayload(data['payload']);
        }
        return job;
    }
    asJson() {
        const obj = {
            'id': this.key,
            'type': this.getType(),
            'tag': this.tag,
            'scheduled_date_time': this.scheduledDateTime.getTime()
        };
        if (!this.payload) {
            return obj;
        }
        obj['payload'] = JSON.stringify(this.payload);
        return obj;
    }
}
Job.TASK_TYPE = 'TASK_TYPE_JOB';
exports.Job = Job;
